@@ -392,6 +392,9 @@ grn_rc grn_db_obj_init(grn_ctx *ctx, grn_obj *db, grn_id id, grn_db_obj *obj);
       break;                                            \
     }                                                   \
     break;                                              \
+  case GRN_VECTOR :                                     \
+    result = GRN_TRUE;                                  \
+    break;                                              \
   default :                                             \
     result = GRN_FALSE;                                 \
     break;                                              \
@@ -446,6 +449,10 @@ GRN_API grn_rc grn_column_filter(grn_ctx *ctx, grn_obj *column,
                                  grn_obj *value, grn_obj *result_set,
                                  grn_operator set_op);
 
+grn_rc grn_accessor_resolve(grn_ctx *ctx, grn_obj *accessor, int deep,
+                            grn_obj *base_res, grn_obj *res, grn_operator op,
+                            grn_search_optarg *optarg);
+
 #ifdef __cplusplus
 }
 #endif
