@@ -20,7 +20,7 @@
 #define GROONGA_IN_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #ifdef __cplusplus
@@ -94,9 +94,6 @@
 # define GRN_WRITE(fd, buf, count) _write(fd, buf, count)
 #endif /* HAVE_WRITE */
 
-GRN_API void grn_sleep(uint32_t seconds);
-GRN_API void grn_nanosleep(uint64_t nanoseconds);
-
 #ifdef WIN32
 
 #if defined(__GNUC__) && !defined(WINVER)
@@ -263,6 +260,13 @@ typedef pthread_mutex_t grn_mutex;
 #define MUTEX_LOCK(m)   pthread_mutex_lock(&m)
 #define MUTEX_UNLOCK(m) pthread_mutex_unlock(&m)
 #define MUTEX_FIN(m)
+#define MUTEX_INIT_SHARED(m) do {\
+  pthread_mutexattr_t mutexattr;\
+  pthread_mutexattr_init(&mutexattr);\
+  pthread_mutexattr_setpshared(&mutexattr, PTHREAD_PROCESS_SHARED);\
+  pthread_mutex_init(&m, &mutexattr);\
+} while (0)
+
 typedef pthread_mutex_t grn_critical_section;
 #define CRITICAL_SECTION_INIT(cs)  pthread_mutex_init(&(cs), NULL)
 #define CRITICAL_SECTION_ENTER(cs) pthread_mutex_lock(&(cs))
@@ -273,6 +277,13 @@ typedef pthread_cond_t grn_cond;
 #define COND_INIT(c)   pthread_cond_init(&c, NULL)
 #define COND_SIGNAL(c) pthread_cond_signal(&c)
 #define COND_WAIT(c,m) pthread_cond_wait(&c, &m)
+#define COND_BROADCAST(c) pthread_cond_broadcast(&c)
+#define COND_INIT_SHARED(c) do {\
+  pthread_condattr_t condattr;\
+  pthread_condattr_init(&condattr);\
+  pthread_condattr_setpshared(&condattr, PTHREAD_PROCESS_SHARED);\
+  pthread_cond_init(&c, &condattr);\
+} while (0)
 
 typedef pthread_key_t grn_thread_key;
 #define THREAD_KEY_CREATE(key, destr)  pthread_key_create(key, destr)
@@ -344,12 +355,14 @@ typedef struct
   HANDLE waiters_count_lock_;
   HANDLE sema_;
   HANDLE waiters_done_;
+  size_t was_broadcast_; 
 } grn_cond;
 
 #define COND_INIT(c) do { \
   (c).waiters_count_ = 0; \
   (c).sema_ = CreateSemaphore(NULL, 0, 0x7fffffff, NULL); \
   MUTEX_INIT((c).waiters_count_lock_); \
+  (c).waiters_done_ = CreateEvent(NULL, FALSE, FALSE, NULL);\
 } while (0)
 
 #define COND_SIGNAL(c) do { \
@@ -363,6 +376,27 @@ typedef struct
   } \
 } while (0)
 
+#define COND_BROADCAST(c) do { \
+  MUTEX_LOCK((c).waiters_count_lock_); \
+  { \
+    int have_waiters = (c).waiters_count_ > 0; \
+    if ((c).waiters_count_ > 0) {  \
+      (c).was_broadcast_ = 1; \
+      have_waiters = 1; \
+    } \
+    if (have_waiters) {\
+      ReleaseSemaphore((c).sema_, (c).waiters_count_, 0); \
+      MUTEX_UNLOCK((c).waiters_count_lock_); \
+      WaitForSingleObject((c).waiters_done_, INFINITE); \
+      (c).was_broadcast_ = 0; \
+    } \
+    else { \
+      MUTEX_UNLOCK((c).waiters_count_lock_); \
+    }\
+  }\
+} while (0)
+      
+  
 #define COND_WAIT(c,m) do { \
   MUTEX_LOCK((c).waiters_count_lock_); \
   (c).waiters_count_++; \
@@ -370,8 +404,16 @@ typedef struct
   SignalObjectAndWait((m), (c).sema_, INFINITE, FALSE); \
   MUTEX_LOCK((c).waiters_count_lock_); \
   (c).waiters_count_--; \
-  MUTEX_UNLOCK((c).waiters_count_lock_); \
-  WaitForSingleObject((m), INFINITE); \
+  { \
+    int last_waiter = (c).was_broadcast_ && (c).waiters_count_ == 0; \
+    MUTEX_UNLOCK((c).waiters_count_lock_); \
+    if (last_waiter)  { \
+      SignalObjectAndWait((c).waiters_done_, (m), INFINITE, FALSE); \
+    } \
+    else { \
+      WaitForSingleObject((m), FALSE); \
+    }\
+  }\
 } while (0)
 
 #else /* WIN32 */
@@ -388,6 +430,9 @@ typedef int grn_cond;
 
 #endif /* WIN32 */
 
+#define MUTEX_INIT_SHARED MUTEX_INIT
+#define COND_INIT_SHARED COND_INIT
+
 #define GRN_TEST_YIELD() do {} while (0)
 
 #endif /* HAVE_PTHREAD_H */
@@ -587,39 +632,81 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 #endif /* WORDS_BIGENDIAN */
 #define grn_ntoh(buf,key,size) grn_hton(buf,key,size)
 
+#ifndef __GNUC_PREREQ
+# if defined(__GNUC__) && defined(__GNUC_MINOR__)
+#  define __GNUC_PREREQ(maj, min) \
+   ((__GNUC__ << 16) + __GNUC_MINOR__ >= ((maj) << 16) + (min))
+# else
+#  define __GNUC_PREREQ(maj, min) 0
+# endif /* defined(__GNUC__) && defined(__GNUC_MINOR__) */
+#endif /* __GNUC_PREREQ */
+
+#ifdef _MSC_VER
+# define grn_bswap_uint64(in, out) ((out) = _byteswap_uint64(in))
+#else /* _MSC_VER */
+# if defined(__GNUC__) && __GNUC_PREREQ(4, 3)
+#  define grn_bswap_uint64(in, out) ((out) = __builtin_bswap64(in))
+# else /* defined(__GNUC__) && __GNUC_PREREQ(4, 3) */
+#  define grn_bswap_uint64(in, out) do {\
+  uint64_t temp_ = (in);\
+  (out) = (temp_ << 56) |\
+          ((temp_ & (0xFFULL << 8)) << 40) |\
+          ((temp_ & (0xFFULL << 16)) << 24) |\
+          ((temp_ & (0xFFULL << 24)) << 8) |\
+          ((temp_ & (0xFFULL << 32)) >> 8) |\
+          ((temp_ & (0xFFULL << 40)) >> 24) |\
+          ((temp_ & (0xFFULL << 48)) >> 40) |\
+          (temp_ >> 56);\
+} while (0)
+# endif /* __GNUC__ */
+#endif /* _MSC_VER */
+
+#ifdef WORDS_BIGENDIAN
+# define grn_hton_uint64(in, out) ((out) = (in))
+#else /* WORDS_BIGENDIAN */
+# define grn_hton_uint64(in, out) grn_bswap_uint64(in, out)
+#endif /* WORDS_BIGENDIAN */
+#define grn_ntoh_uint64(in, out) grn_hton_uint64(in, out)
+
 #define grn_gton(keybuf,key,size) do {\
-  uint8_t *keybuf_ = keybuf;\
-  const void *key_ = key;\
-  int la = ((grn_geo_point *)key_)->latitude;\
-  int lo = ((grn_geo_point *)key_)->longitude;\
-  uint8_t *p = keybuf_;\
-  int i = 32;\
-  while (i) {\
-    i -= 4;\
-    *p++ = ((((la >> i) & 8) << 4) + (((lo >> i) & 8) << 3) +\
-            (((la >> i) & 4) << 3) + (((lo >> i) & 4) << 2) +\
-            (((la >> i) & 2) << 2) + (((lo >> i) & 2) << 1) +\
-            (((la >> i) & 1) << 1) + (((lo >> i) & 1) << 0));\
-  }\
+  const grn_geo_point *point_ = (const grn_geo_point *)key;\
+  uint64_t la_ = (uint32_t)point_->latitude;\
+  uint64_t lo_ = (uint32_t)point_->longitude;\
+  uint64_t result_;\
+  la_ = (la_ | (la_ << 16)) & 0x0000FFFF0000FFFFULL;\
+  la_ = (la_ | (la_ <<  8)) & 0x00FF00FF00FF00FFULL;\
+  la_ = (la_ | (la_ <<  4)) & 0x0F0F0F0F0F0F0F0FULL;\
+  la_ = (la_ | (la_ <<  2)) & 0x3333333333333333ULL;\
+  la_ = (la_ | (la_ <<  1)) & 0x5555555555555555ULL;\
+  lo_ = (lo_ | (lo_ << 16)) & 0x0000FFFF0000FFFFULL;\
+  lo_ = (lo_ | (lo_ <<  8)) & 0x00FF00FF00FF00FFULL;\
+  lo_ = (lo_ | (lo_ <<  4)) & 0x0F0F0F0F0F0F0F0FULL;\
+  lo_ = (lo_ | (lo_ <<  2)) & 0x3333333333333333ULL;\
+  lo_ = (lo_ | (lo_ <<  1)) & 0x5555555555555555ULL;\
+  result_ = (la_ << 1) | lo_;\
+  grn_hton_uint64(result_, result_);\
+  memcpy(keybuf, &result_, sizeof(result_));\
 } while (0)
 
 #define grn_ntog(keybuf,key,size) do {\
-  uint8_t *keybuf_ = keybuf;\
-  uint8_t *key_ = key;\
-  uint32_t size_ = size;\
-  int la = 0, lo = 0;\
-  uint8_t v, *p = key_;\
-  int i = 32;\
-  while (size_--) {\
-    i -= 4;\
-    v = *p++;\
-    la += (((v & 128) >> 4) + ((v &  32) >> 3) +\
-           ((v &   8) >> 2) + ((v &   2) >> 1)) << i;\
-    lo += (((v &  64) >> 3) + ((v &  16) >> 2) +\
-           ((v &   4) >> 1) + ((v &   1) >> 0)) << i;\
-  }\
-  ((grn_geo_point *)keybuf_)->latitude = la;\
-  ((grn_geo_point *)keybuf_)->longitude = lo;\
+  grn_geo_point *point_ = (grn_geo_point *)keybuf;\
+  uint64_t key_ = *(const uint64_t *)key;\
+  uint64_t la_, lo_;\
+  grn_ntoh_uint64(key_, key_);\
+  la_ = (key_ >> 1) & 0x5555555555555555ULL;\
+  lo_ = key_ & 0x5555555555555555ULL;\
+  la_ = (la_ | (la_ >>  1)) & 0x3333333333333333ULL;\
+  la_ = (la_ | (la_ >>  2)) & 0x0F0F0F0F0F0F0F0FULL;\
+  la_ = (la_ | (la_ >>  4)) & 0x00FF00FF00FF00FFULL;\
+  la_ = (la_ | (la_ >>  8)) & 0x0000FFFF0000FFFFULL;\
+  la_ = (la_ | (la_ >> 16)) & 0x00000000FFFFFFFFULL;\
+  lo_ = (lo_ | (lo_ >>  1)) & 0x3333333333333333ULL;\
+  lo_ = (lo_ | (lo_ >>  2)) & 0x0F0F0F0F0F0F0F0FULL;\
+  lo_ = (lo_ | (lo_ >>  4)) & 0x00FF00FF00FF00FFULL;\
+  lo_ = (lo_ | (lo_ >>  8)) & 0x0000FFFF0000FFFFULL;\
+  lo_ = (lo_ | (lo_ >> 16)) & 0x00000000FFFFFFFFULL;\
+  point_->latitude = la_;\
+  point_->longitude = lo_;\
 } while (0)
 
 #ifndef HAVE_STRTOULL
@@ -663,6 +750,9 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 #endif /* POSIX_HOST_NAME_MAX */
 #endif /* HOST_NAME_MAX */
 
+GRN_API void grn_sleep(uint32_t seconds);
+GRN_API void grn_nanosleep(uint64_t nanoseconds);
+
 #ifndef GROONGA_H
 #include "groonga.h"
 #endif /* GROONGA_H */
